#!/usr/bin/python
# -*- coding: utf-8 -*-
####
#### Just a deck of whatever.
####
#### TODO: use shallow copy for incoming args for easier capture?
#### TODO: multi-dimensional cards? e.g. not just ["front", "back"]...
####

import random

## The first is always the "main" one.
class DeckManager:

    def __init__(self, in_subdeck_list=["main"],
                 in_facets_by_face={'front': {}, 'back': {}}):

        self.total_count = 0
        self.subdeck_list = in_subdeck_list
        self.face_list = ["front", "back"]
        self.subdecks = {}
        self.main_deck_name = None
        self.facets_by_face = in_facets_by_face

        ## Always at least "main".
        if len(self.subdeck_list) == 0:
            self.subdeck_list.append("main")

        ##
        self.main_deck_name = self.subdeck_list[0]
        for subdeck in self.subdeck_list:
            self.subdecks[subdeck] = []

    def add(self, card, target=None):
        "Docstring..."
        if not target: target = self.main_deck_name
        self.subdecks[target].append(card)
        self.total_count = self.total_count + 1

    # Shuffles the play deck.
    def shuffle(self, target=None):
        if not target: target = self.main_deck_name
        random.shuffle(self.subdecks[target])

    # Merge subdecks back into a single deck.
    def merge(self, src=None, dst=None):
        if not src: src = self.main_deck_name

        ## Check arg number; if only one, src is main and arg is dst.
        if not dst:
            dst = src
            src = self.main_deck_name

        if src == dst:
            pass
        else:
            self.subdecks[dst].extend(self.subdecks[src])
            self.subdecks[src] = []

    # Merge subdecks back into a single deck.
    def merge_all(self, target=None):
        if not target: target = self.main_deck_name
        for subdeck in self.subdeck_list:
            if subdeck == target:
                pass
            else:
                self.subdecks[target].extend(self.subdecks[subdeck])
                self.subdecks[subdeck] = []

    # Switch two decks.
    def switch(self, deck_a, deck_b):
        if deck_a in self.subdeck_list and deck_b in self.subdeck_list:
            tmp_a = self.subdecks[deck_a]
            tmp_b = self.subdecks[deck_b]
            self.subdecks[deck_a] = tmp_b
            self.subdecks[deck_b] = tmp_a

    # Current card at "top" of deck.
    def top(self, target=None):
        if not target: target = self.main_deck_name
        try:
            #print "", 
            return self.subdecks[target][-1]
        except:
            return None        
        return None

    ## Return the face (defined in facets) of the target card.
    def see_top(self, face_name=None, target=None):
        if not target: target = self.main_deck_name
        result_hash = None
        #print "face_list:", self.face_list
        if face_name in self.face_list:

            #print "face_name:", face_name
            test_hash = self.facets_by_face[face_name]
            #print "test_hash:", test_hash
            result_hash = {}
            #print "target:", target
            ccard = self.top(target)
            #print "ccard:", ccard
            if ccard:
                for key in test_hash.keys():
                    if test_hash.get(key):
                        value = ccard.get(key)
                        if value:
                            # print "__face: adding:", key, value
                            result_hash[key] = value
        # print "__face: result_hash:", result_hash
        return result_hash
        
    def move_top(self, src=None, dst=None):
        if not src: src = self.main_deck_name

        ## Check arg number; if only one, src is main and arg is dst.
        if not dst:
            dst = src
            src = self.main_deck_name

        ## Do it.
        src_card_test = self.top(src)
        src_card = None
        if src_card_test:
            src_card = self.subdecks[src].pop()
            self.subdecks[dst].append(src_card)

    # TODO
    def count(self, target=None):
        count_res = None
        if not target:
            count_res = self.total_count
        elif target in self.subdeck_list:
            count_res = len(self.subdecks[target])
        return count_res

###
### Self-test.
###

import unittest


# TODO:
#class TestsingleDeck(unittest.TestCase):


class TestTripleDeck(unittest.TestCase):

    # Unused before each.
    def setUp(self):
        pass

    # Unused after each.
    def tearDown(self):
        pass

    # Used before all in class.
    @classmethod
    def setUpClass(cls):
        pass

    # Unused before all in class.
    @classmethod
    def tearDownClass(cls):
        pass

    # We want to run these in order, so not the usual layout
    # (different tests defined by test_*).

    def runTest(self):

        #
        # Setup.
        #

        tmpl = {'front': {'foo': True, 'bar': True, 'bibble': False},
                'back': {'foo': False, 'bar': True, 'bibble': True}}
        self.deck = DeckManager(["play", "right", "wrong"], tmpl)

        cards = [{'foo': 1, 'bar': 'a', 'bibble': ['al', 'pha']},
                 {'foo': 2, 'bar': 'b', 'bibble': ['be', 'ta']},
                 {'foo': 3, 'bar': 'c', 'bibble': ['gam', 'ma']},
                 {'foo': 4, 'bar': 'd', 'bibble': ['del', 'ta']},
                 {'foo': 5, 'bar': 'e', 'bibble': ['epsi', 'lon']},
                 {'foo': 6, 'bar': 'f', 'bibble': ['ze', 'ta']}]
        for card in cards:
            self.deck.add(card)

        #
        # Initial counts.
        #

        self.assertEqual(self.deck.count(), 6)
        self.assertEqual(self.deck.count("play"), 6)
        self.assertEqual(self.deck.count("right"), 0)
        self.assertEqual(self.deck.count("wrong"), 0)

        #
        # Sub-deck juggling with switch_with_reserve.
        #

        # Reflexive juggle.
        self.deck.switch("play", "wrong")
        self.deck.switch("play", "wrong")
        self.assertEqual(self.deck.count(), 6)
        self.assertEqual(self.deck.count("play"), 6)
        self.assertEqual(self.deck.count("right"), 0)
        self.assertEqual(self.deck.count("wrong"), 0)

        # Switch.
        self.deck.switch("play", "wrong")
        self.assertEqual(self.deck.count(), 6)
        self.assertEqual(self.deck.count("play"), 0)
        self.assertEqual(self.deck.count("right"), 0)
        self.assertEqual(self.deck.count("wrong"), 6)

        # And back.
        self.deck.switch("wrong", "play")
        self.assertEqual(self.deck.count(), 6)
        self.assertEqual(self.deck.count("play"), 6)
        self.assertEqual(self.deck.count("right"), 0)
        self.assertEqual(self.deck.count("wrong"), 0)

        #
        # Simple activity.
        #

        seen_1 = self.deck.see_top("front", "play")
        #print "s1:", seen_1
        foo_1 = seen_1['foo']
        
        self.deck.move_top("play", "wrong")
        self.assertEqual(self.deck.count("play"), 5)
        self.assertEqual(self.deck.count("right"), 0)
        self.assertEqual(self.deck.count("wrong"), 1)
        
        seen_2 = self.deck.see_top("front", "wrong")
        foo_2 = seen_2['foo']

        self.assertEqual(foo_1, foo_2)
        
        #
        # More activity.
        #

        self.deck.move_top("play", "wrong")
        self.deck.move_top("play", "right")
        self.assertEqual(self.deck.count("play"), 3)
        self.assertEqual(self.deck.count("right"), 1)
        self.assertEqual(self.deck.count("wrong"), 2)        

        self.deck.switch("wrong", "right")
        self.assertEqual(self.deck.count("play"), 3)
        self.assertEqual(self.deck.count("wrong"), 1)
        self.assertEqual(self.deck.count("right"), 2)        

        self.deck.merge_all("right")
        self.assertEqual(self.deck.count("play"), 0)
        self.assertEqual(self.deck.count("wrong"), 0)
        self.assertEqual(self.deck.count("right"), 6)        

        #
        # Deck exhaustion.
        #

        for r in range(0, 20):
            #print "r:", r
            self.deck.move_top("right", "play")
        self.assertEqual(self.deck.count("play"), 6)
        self.assertEqual(self.deck.count("wrong"), 0)
        self.assertEqual(self.deck.count("right"), 0)        

        #
        # Single merge.
        #

        self.deck.move_top("play", "wrong")
        self.deck.move_top("play", "wrong")
        self.deck.move_top("play", "wrong")
        self.deck.move_top("play", "right")
        self.deck.move_top("play", "right")
        self.deck.move_top("play", "right")
        self.assertEqual(self.deck.count("play"), 0)
        self.assertEqual(self.deck.count("wrong"), 3)
        self.assertEqual(self.deck.count("right"), 3)

        self.deck.merge("wrong", "play")
        self.assertEqual(self.deck.count("play"), 3)
        self.assertEqual(self.deck.count("wrong"), 0)
        self.assertEqual(self.deck.count("right"), 3)
        

if __name__ == "__main__":

    unittest.main()
