////
//// quiz-bootstrap.js
////
//// Assume the existance of the globals:
////    wrenshoe. ...
////    .current_deck
////    .deck_list
////    etc...
////
//// To make this work, there were some coding problems.
//// Apparently, JS does not really do UTF-8, making some things
//// a hideous pain. See:
////  http://mathiasbynens.be/notes/javascript-encoding
//// To get around this, once the data has been been pulled in from the utf-8
//// sources (editted in emacs), the output is in utf-16, which
//// apparently is more digestable (at least my characters are coming
//// out as not garbage).
//// See ../compile-directory.pl for more details.
//// 

// For debugging.
var _logger = new bbop.logger('Q: ');
_logger.DEBUG = true;
//_logger.DEBUG = false;
function ll(str){
    _logger.kvetch(str);
}

// // 
// var anchor = this;

var wrenshoe_session_type = null;
var wrenshoe_session_id = null;
var wrenshoe_session_interval = 3000;

// Livable aliases. 
var each = bbop.core.each;
var clone = bbop.core.clone;
var dump = bbop.core.dump;
var what_is = bbop.core.what_is;
var is_defined = bbop.core.is_defined;
var uuid = bbop.core.uuid;

//
var main_div_id = "#wrenshoe-main";
var nav_div_id = "#wrenshoe-nav";

// The landing HTML we will override when the state machine starts.
// This will be captured when QuizInit starts.
var global_init_landing_html = '<p>Error in start sequence!</p>';

///
/// Helper writers.
///

function _append(str){
    jQuery(main_div_id).append(str);
}

function _head(str){
    return '<h1>' + str + '</h1>';
}
function _text(str){
    return '<p class="lead">' + str + '</p>';
    //jQuery(main_div_id).append('<p class="">' + str + '</p>');
}
function _input_box(label, value, name, class_str, type, checked_p){
    var fstr = '';
    fstr += '<div class="' + type + '">';
    fstr += '<label>';
    fstr += '<input ';
    fstr += 'type="' + type + '" ';
    fstr += 'name="' + name + '" ';
    fstr += 'class="' + class_str + '" ';
    fstr += 'value="' + value + '" ';
    if( checked_p ){
	fstr += 'checked="checked" ';
    }
    fstr += '>';
    fstr += label;
    fstr += '</label>';
    fstr += '</div>';

    return fstr;
}
function _panel(str){
    var fstr = '';
    fstr += '<div class="panel panel-default">';
    fstr += '<div class="panel-body">';
    fstr += str;
    fstr += '</div>';
    fstr += '</div>';

    return fstr;
}

function _draw_deck_faces_at(deck, faces){

    ll('ddfa: ' + faces.length);

    var str = '';

    // First, determine if and what tags to show.
    // This shows all tags in natural order.
    if( wrenshoe.current_meta_show_p ){

	// Determine tag information, if any.
	var tag_line = '';
	var ctags = deck.look_at_tags();
	if( ctags && ctags.length > 0 ){
	    tag_line = '<br />tags: ' + ctags.join(', ');
	}

	// Give a status update.
	str += '<p><small>' +
	    'todo: ' + deck.number_of_cards('todo') +
	    ', done: ' + deck.number_of_cards('done') +
	    ', redo: ' + deck.number_of_cards('redo') +
	    ', filtered: ' + deck.number_of_cards('filtered') +
	    tag_line +
	    '</small></p>';
    }

    // Display whatever we have in the content section.
    var view = deck.look_at_card(faces);
    each(view,
	 function(vitem){
	     str += '<p><big><b>' + vitem + '</b></big></p>';
	 });
    
    return str;
}

///
/// Transition actions.
///

function _advance_deck_done(){
    var deck = wrenshoe.current_deck;
    deck.done();
}

function _advance_deck_redo(){
    var deck = wrenshoe.current_deck;
    deck.redo();
}

function _session_quiz(){
    wrenshoe_session_type = 'quiz';
}
function _session_slides(){
    wrenshoe_session_type = 'slides';    
}

function _cancel_slideshow(){
    if( wrenshoe_session_id ){
	window.clearTimeout(wrenshoe_session_id);
	wrenshoe_session_type = null; //remove type string
	wrenshoe_session_id = null;  // remove if int
    }
}

///
/// Transitions tests.
///

function _is_slides_session(){
    var ret = false;
    if( wrenshoe_session_type == 'slides' ){
	ret = true;
    }
    return ret;
}

function _no_more_cards(){
    var ret = false;
    if( is_defined(wrenshoe) &&
	is_defined(wrenshoe.current_deck) &&
	wrenshoe.current_deck.is_empty() ){
	ret = true;
    }
    return ret;
}

///
/// Transitions tests.
///

function _action_redo_all(){
    var deck = wrenshoe.current_deck;
    var pa = deck.positive_filter_list();
    deck.reset();
    deck.shuffle();
    deck.apply_positive_filters(pa);
}

function _action_redo_missed(){	    
    var deck = wrenshoe.current_deck;
    deck.merge_into('redo', 'todo');
    deck.shuffle();
}

///
/// Page renderers.
///

function _render_welcome(){
    // Try and preserve pre-rendered HTML that we start with.
    return global_init_landing_html;
}

// Render the quiz selection page.
function _render_select(){

    // First, the text that's always there.
    _append(_head('Select your self-quiz set!'));
    _append(_text('Select a self-quiz from the list, then select either "Review" or "Self-quiz" from the bottom.'));
    _append(_text('You can return to this page at any time by pressing the "Home" button in the upper right or the "Reselect" button at other times.'));

    // Now, create a list from the wrenshoe.data array that we should
    // have in our environment.
    var dlist = _get_deck_list(wrenshoe.deck_list);

    // If we found any data, make a list, otherwise write a sad
    // message.
    if( dlist.length == 0 ){
	ll('data list empty');
	_append(_text('<b>Sorry, no data was found!</b>'));
    }else{

	// We have data, so let's add it to our world.
	var sids = [];
	var inputs = [];
	each(dlist,
	     function(datum, index){
		 sids.push(datum['id']); // save for later activation

		 var lbl = datum['name'];
		 var val = datum['id'];
		 if( index == 0 ){
		     inputs.push(_input_box(lbl, val, "select-input",
					    "select-group", "radio",
					    true));
		 }else{
		     inputs.push(_input_box(lbl, val, "select-input",
					    "select-group", "radio",
					    false));
		 }

		 // The starter deck in case we leave quickly.
		 wrenshoe.current_deck =
		     _get_deck(wrenshoe.deck_list, datum['id']);
	     });

	// Actually add all the inputs to the DOM in a panel.
	_append(_panel(inputs.join('')));

	// Get the current checked value.
	function _pull_values(){
	    var vals = _get_checked_values('select-input');
	    if( ! vals || ! vals[0] ){
		alert('Clicked, but no value found to review!');
	    }else{
		var quiz_id = vals[0];
		ll('click! ' + quiz_id);
		wrenshoe.current_deck =
		    _get_deck(wrenshoe.deck_list, quiz_id);
	    }
	}

	// Initialize with the current value.
	_pull_values();

	// Update the current value whenever clicked.
	jQuery(".select-group").click(
	    function(evnt){
		evnt.stopPropagation();
		_pull_values();
	    });
    }

    return null;
}

function _render_review(){

    // We're assuming that wrenshoe.current_deck is set.
    var deck = wrenshoe.current_deck;

    var tbl = '';
    if( ! deck ){
	ll('error get_deck');
    }else{
	
	tbl +='<div class="table-responsive">';
	tbl +='<table class="table table-striped table-hover table-condensed">';
	tbl +='<thead>';
	tbl +='<tr>';

	// Group all tags under one heading...for now.
	var cols = clone(deck.faces());
	cols.push('Tags');
	each(cols,
	     function(col){
		 tbl += '<td>';
		 tbl += col;
		 tbl += '</td>';
	     });

	tbl += '</tr>';
	tbl += '</thead>';
	tbl += '<tbody>';

	var lcache = [];
	while( ! deck.is_empty() ){
	    
	    var writes = clone(deck.look_at_card(deck.faces()));
	    writes.push(deck.look_at_tags().join(', '));
	    
	    lcache.push(writes);

	    // Push stop condition.
	    deck.done();
	}

	lcache.reverse();
	each(lcache,
	     function(line){
		 tbl += '<tr>';
		 each(line,
		      function(item){
			  tbl += '<td>';
			  tbl += item;
			  tbl += '</td>';
		      });
		 tbl += '</tr>';
	     });

	tbl += '</tbody>';
	tbl += '</table>';
	tbl += '</div>';
    }

    return tbl;
}

function _render_faces(){

    var deck = wrenshoe.current_deck;

    // Build up the front and back.
    var faces = deck.faces();

    // Whenever we touch anything, we change the state of the whole
    // system.
    function _make_state(evnt){
	if( evnt ){
	    evnt.stopPropagation();	    
	}
	    
	// Get the faces that we'll use.
	wrenshoe.current_front_faces =
	    _get_checked_values('front-face-input');
	wrenshoe.current_back_faces =
	    _get_checked_values('back-face-input');
	wrenshoe.current_filters =
	    _get_checked_values('tag-positive-input');
	    
	// Also get the "optional" show tags flag.
	var tsi_list = _get_checked_values('meta-show-input');	
	wrenshoe.current_meta_show_p = false; // hyper default
	if( is_defined(tsi_list) && what_is(tsi_list) == 'array' &&
	    tsi_list.length == 1 ){
		var tsi = tsi_list[0];
		if( what_is(tsi) == 'string' && tsi == 'show' ){
		    wrenshoe.current_meta_show_p = true;
		}
	    }
	
	// Shuffle once before we go in.
	wrenshoe.current_deck.reset();
	wrenshoe.current_deck.shuffle();

	// Apply any filters we might have just picked up.
	var pf = wrenshoe.current_filters;
	wrenshoe.current_deck.apply_positive_filters(pf);

	ll('finished faces state change');
    }

    ///
    /// Intro section.
    ///

    if( wrenshoe_session_type == 'quiz' ){
	_append(_head('Quiz: choose the faces for each side!'));
    }else if( wrenshoe_session_type == 'slides' ){
	_append(_head('Slides: choose the faces for each side!'));
    }else{
	throw new Error('unknown session');
    }

    ///
    /// Front faces section.
    ///

    _append(_text('Choose the faces that you want to see on the front.'));

    // Front face possibilities.
    var finputs = [];
    each(faces,
	 function(face, index){
	     // Add literal.
	     if( index == 0 ){
		 finputs.push(_input_box(face, face, "front-face-input",
					 "face-inputs", "checkbox", true));
	     }else{
		 finputs.push(_input_box(face, face, "front-face-input",
					 "face-inputs", "checkbox", false));
	     }
	 });
    _append(_panel(finputs.join('')));

    ///
    /// Back faces section.
    ///

    _append(_text('Choose the faces that you want to see on the back.'));

    // Back face possibilities.
    var binputs = [];
    each(faces,
	 function(face, index){
	     binputs.push(_input_box(face, face, "back-face-input",
				     "face-inputs", "checkbox", true));
	 });
    _append(_panel(binputs.join('')));

    ///
    /// Meta section.
    ///

    _append(_text('Choose whether to additionally show information about the deck and current card during the quiz.'));
    _append(_panel(_input_box("Show deck and tag information", "show",
			      "meta-show-input", "face-inputs", "checkbox",
			      true)));

    ///
    /// Tag section.
    ///

    // Now build up tags.
    var tags = deck.tag_list();
    if( ! tags || ! tags.length || tags.length == 0 ){
	// If no tags are defined, don't render whole section.
    }else{
	// Otherwise, we're going to build up the inputs.

	//
	_append(_text("Choose the tags that you want to review. If you don't select any tags, all cards will be viewed. The default is to view all cards."));

	// Select/unselect all.
	var sall = '';
	sall += '<div class="btn-group">';
	sall += '<button id="select-all-tags" type="button" class="btn btn-default">Select all</button>';
	sall += '<button id="unselect-all-tags" type="button" class="btn btn-default">Unselect all</button>';
	sall += '</div>';
	jQuery(main_div_id).append(sall);
	// Make the select all and unselect all buttons active on the
	// faces page.
	var tag_space = 'input[name="tag-positive-input"]';
	jQuery("#select-all-tags").click(
	    function(evnt){
		evnt.stopPropagation();
		jQuery(tag_space).attr('checked', 'checked');
		_make_state(evnt);
		ll('select happened');
	    });
	jQuery("#unselect-all-tags").click(
	    function(evnt){
		evnt.stopPropagation();
		jQuery(tag_space).removeAttr('checked');
		_make_state(evnt);
		ll('unselect happened');
	    });
	
	// Write out the tags.
	tags.sort(); // just this sort for now
	var tinputs = [];
	each(tags,
	     function(tag){
		 tinputs.push(_input_box(tag, tag, "tag-positive-input",
					 "face-inputs", "checkbox", false));
	     });

	_append(_panel(tinputs.join('')));
    }

    // Add events to all face inputs.
    jQuery('.face-inputs').click(_make_state);

    // ...and initialize with a first run.
    _make_state(null);

    return null;
}

function _render_slides_front(){

    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_front_faces;

    // Deck emptiness should be a front-only problem debugging
    // problem. If there are no cards, let's just announce it for now.
    if( deck.is_empty() ){
	ll('front: deck is empty');
	_append(_text('EMPTY'));
    }else{
	_append(_draw_deck_faces_at(deck, faces));
    }

    // Setup timer if there is not already one.
    if( wrenshoe_session_id != null ){
	window.clearTimeout(wrenshoe_session_id);
    }
    wrenshoe_session_id =
	window.setTimeout(function(){ _change_page('slides_back'); }, 
			  wrenshoe_session_interval);

    return null;
}

function _render_slides_back(){

    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_back_faces;

    _append(_draw_deck_faces_at(deck, faces));

    // Reshuffle the deck.
    deck.shuffle();

    // Setup timer if there is not already one.
    if( wrenshoe_session_id != null ){
	window.clearTimeout(wrenshoe_session_id);
    }
    wrenshoe_session_id =
	window.setTimeout(function(){ _change_page('slides_front'); },
			  wrenshoe_session_interval);

    return null;
}

function _render_quiz_front(){

    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_front_faces;

    // Deck emptiness should be a front-only problem debugging
    // problem. If there are no cards, let's just announce it for now.
    if( deck.is_empty() ){
	ll('front: deck is empty');
	_append(_text('EMPTY'));
    }else{
	_append(_draw_deck_faces_at(deck, faces));
    }

    return null;
}

function _render_quiz_back_positive(){

    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_back_faces;

    _append(_draw_deck_faces_at(deck, faces));

    return null;
}

function _render_quiz_back_negative(){

    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_back_faces;

    _append(_draw_deck_faces_at(deck, faces));

    return null;
}

function _render_quiz_final(){

    var deck = wrenshoe.current_deck;
    _append('<p>todo: ' + deck.number_of_cards('todo') + '</p>' +
	    '<p>redo: ' + deck.number_of_cards('redo') + '</p>' +
	    '<p>done: ' + deck.number_of_cards('done') + '</p>' +
	    '<p>filtered: ' + deck.number_of_cards('filtered') + '</p>');
	
    return null;
}

///
/// Page definitions.
///

var pages = {
  'welcome': {
      'actions': [],
      'content': _render_welcome,
      'transitions': [
	  {
	      'title': 'This way to begin!',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'select'
		  }
	      ]
	  }
      ]
  },
  'select': {
      'actions': [],
      'content': _render_select,
      'transitions': [
	  {
	      'title': 'Review',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'review'
		  }
	      ]
	  },
	  {
	      'title': 'Slides',
	      'actions': [_session_slides],
	      'conditions': [
		  {
		      'test': null,
		      'target': 'faces'
		  }
	      ]
	  },
	  {
	      'title': 'Quiz',
	      'actions': [_session_quiz],
	      'conditions': [
		  {
		      'test': null,
		      'target': 'faces'
		  }
	      ]
	  }
      ]
  },
  'review': {
      'actions': [],
      'content': _render_review,
      'transitions': [
	  {
	      'title': 'Return to selection',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'select'
		  }
	      ]
	  }
      ]
  },
  'faces': {
      'actions': [],
      'content': _render_faces,
      'transitions': [
	  {
	      'title': 'Re-select',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'select'
		  }
	      ]
	  },
	  {
	      'title': 'Start',
	      'conditions': [
		  {
		      'test': _is_slides_session,
		      'target': 'slides_front'
		  },
		  {
		      'test': null,
		      'target': 'quiz_front'
		  }
	      ]
	  }
      ]
  },
  'slides_front': {
      'actions': [],
      'content': _render_slides_front,
      'transitions': [
	  {
	      'title': 'Return to selection',
	      'conditions': [
		  {
		      'test': null,
		      'actions': [_cancel_slideshow],
		      'target': 'select'
		  }
	      ]
	  }
      ]
  },
  'slides_back': {
      'actions': [],
      'content': _render_slides_back,
      'transitions': [
	  {
	      'title': 'Return to selection',
	      'conditions': [
		  {
		      'test': null,
		      'actions': [_cancel_slideshow],
		      'target': 'select'
		  }
	      ]
	  }
      ]
  },
  'quiz_front': {
      'actions': [],
      'content': _render_quiz_front,
      'transitions': [
	  {
	      'title': 'Okay',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'quiz_back'
		  }
	      ]
	  }
      ]
  },
  'quiz_back': {
      'actions': [],
      'content': _render_quiz_back_positive,
      'transitions': [
  	  {
  	      'title': 'Yes',
  	      'type': 'success',
	      'actions': [_advance_deck_done],
  	      'conditions': [
		  {
		      'test': _no_more_cards,
		      'target': 'quiz_final'
		  },
  		  {
  		      'test': null,
  		      'target': 'quiz_front'
  		  }
  	      ]
  	  },
  	  {
  	      'title': 'No',
  	      'type': 'danger',
	      'actions': [_advance_deck_redo],
  	      'conditions': [
		  {
		      'test': _no_more_cards,
		      'target': 'quiz_final'
		  },
  		  {
  		      'test': null,
  		      'target': 'quiz_front'
  		  }
  	      ]
  	  }
      ]
  },
  'quiz_final': {
      'actions': [],
      'content': _render_quiz_final,
      'transitions': [
	  {
	      'title': 'Re-select quiz',
	      'conditions': [
		  {
		      'test': null,
		      'target': 'select'
		  }
	      ]
	  },
	  {
	      'title': 'Redo quiz',
	      'conditions': [
		  {
		      'test': null,
		      'actions': [_action_redo_all],
		      'target': 'quiz_front'
		  }
	      ]
	  },
	  {
	      'title': 'Redo missed',
	      'conditions': [
		  {
		      'test': null,
		      'actions': [_action_redo_missed],
		      'target': 'quiz_front'
		  }
	      ]
	  }
      ]
  }
};

// Works off of the current global state/state machine.
function _change_page(target){
    
    if( ! is_defined(pages[target]) ){
	throw new Error('unknown page target: ' + target);
    }else{
	
	// Setup.
	var page = pages[target];

	// Run pre-run functions.
	var pre_actions = page['actions'] || [];
	each(pre_actions,
	     function(pre_run_function){
		 pre_run_function.apply(this, null); // context, arglist
	     });

	// Wipe out current text with renderer text.
	// It will either self-render or return a stringthat we
	// render.
	jQuery(main_div_id).empty();
	var content = page['content'].apply(this, null);
	if( content ){
	    jQuery(main_div_id).append(content);
	}

	// Wipe out buttons and replace with new transitions.
	jQuery(nav_div_id).empty();
	each(page['transitions'],
	     function(btn){

		 var title = btn['title'];
		 var type = btn['type'] || 'primary';
		 var button_actions = btn['actions'] || [];
		 var conditions = btn['conditions'] || [];

		 // Create the button string, including a generated
		 // ID; add it to the DOM.
		 var bid = uuid();
		 var bstr = '<a id="' +
		     bid + '" href="#" class="btn btn-' +
		     type + ' btn-lg navbar-btn" role="button">' +
		     title + '</a>';
		 jQuery(nav_div_id).append(bstr);

		 // Add a function to the button click that will scan
		 // the conditions list for a true testing function,
		 // or null (which should be last, as it is "default"
		 // or fallthrough).
		 jQuery('#' + bid).click(
		     function(evnt){
			 evnt.stopPropagation();

			 // Relying on break behavior here, so cannot
			 // use usual iterator.
			 for( var ai = 0; ai < button_actions.length; ai++ ){
			     var action = button_actions[ai];
			     action.apply(this, null);
			 }

			 // Ant now see if we get any transitions
			 // along the way.
			 var found_transition_p = false;
			 for( var ci = 0; ci < conditions.length; ci++ ){
			     var condition = conditions[ci];

			     var test = condition['test'] || null;
			     var cond_actions = condition['actions'] || [];
			     var target = condition['target'];
			     
			     // Prepare to run the action before the
			     // transition.
			     var calen = cond_actions.length;
			     function _run_actions(){
				 for( var cai = 0; cai < calen; cai++ ){
				     var action = cond_actions[cai];
				     action.apply(this, null);
				 }
			     }

			     if( test == null ){ // default case of no test
				 _run_actions();
				 _change_page(target);
				 found_transition_p = true;
				 break;
			     }else{
				 var test_ret = test.apply(this, null);
				 if( test_ret && test_ret == true ){
				     _run_actions();
				     _change_page(target);
				     found_transition_p = true;
				     break;
				 }
			     }
			 }
			 // Hopefully we got a transition.
			 if( ! found_transition_p ){
			     ll('Error: no transition found!');
			 }
		     }
		 );
	     });
    }
}

///
/// Helper data functions.
///

// TODO: Make less dumb.
// Create a list from the wrenshoe.data array that we should
// have in our environment.
function _get_deck_list(deck_list){
    var found_decks = [];
    each(deck_list,
		   function(datum, index){
		       var d = {
			   id: datum['id'],
			   name: datum['name'],
			   index: index 
		       };
		       found_decks.push(d);
		   });
    return found_decks;
}

function _get_deck(deck_list, deck_id){
    
    var retdeck = null;

    each(deck_list,
		   function(deck_to_check){
		       if( deck_id == deck_to_check['id'] ){
			   retdeck = deck_to_check;
		       }
		   });
    
    // If we found it, convert it to our Deck.
    if( retdeck ){
	retdeck = new Deck(retdeck);
    }

    return retdeck;
}

function _get_first_deck(deck_list){

    var retdeck = null;

    // Something there?
    if( deck_list && deck_list.length && deck_list[0] ){
	retdeck = deck_list[0];
    }
    
    // If we found it, convert it to our Deck.
    if( retdeck ){
	retdeck = new Deck(retdeck);
    }

    return retdeck;
}

function _get_checked_values(checkbox_name){
    var checked_vals = [];
    jQuery("[name='" + checkbox_name + "']:checked").each(
	function(){
	    checked_vals.push(jQuery(this).val());
	});
    return checked_vals;
}

///
/// Main page control fuunctions.
///

function QuizInit(){
    ll('QuizInit');

    global_init_landing_html = jQuery(main_div_id).html();

    // Okay, special handling depending on whether or not there are
    // multiple decks or just a single one.
    if( ! wrenshoe.deck_list || ! wrenshoe.deck_list.length ){
	ll('Global deck list is not well defined...very bad!');
	throw new Error('Huh!? No deck list...');
    }else{
	var deck_count = wrenshoe.deck_list.length;
	ll('There are ' + deck_count + ' decks defined.');

	if( deck_count == 1 ){
	    //ll('Using single mode.');
	    ll('Sorry--single mode not yet available!');

	    // TODO
	    //throw new Error('Sorry--single mode not yet available!');
	    _change_page('welcome');

	}else{
	    ll('Using multi mode.');

	    _change_page('welcome');
	}
    }
}

// Let's just wait until everything is settled. Just once.
jQuery(document).ready(function(){ QuizInit(); });
