////
//// deck.js
////

function Deck(json_deck_data){

    // Aliases.
    var each = bbop.core.each;

    ///
    /// Initialization (pre def).
    /// NOTE: There is also a couple of things at the very very end.
    ///

    // Remove the incoming arguments.
    var id = json_deck_data['id'];
    var name = json_deck_data['name'];
    var columns = json_deck_data['columns'];
    var cards = json_deck_data['cards'];
    var tag_fields = json_deck_data['tag_fields'];
    var tags = json_deck_data['tags'];

    // Figure out and index the faces and columns.
    var faces_list = []; 
    each(columns,
	 function(col){
	     if( col['type'] == 'face' ){
		 faces_list.push(col['name']);
	     }
	 });
    var columns_list = []; 
    each(columns,
	 function(col){
	     columns_list.push(col['name']);
	 });

    // Filters.    
    var positive_filter_list = []; 

    // Instead of really shuffling the cards and the like, just create
    // some references for them.
    var card_index =
	{
	    todo: [],
	    redo: [],
	    done: [],
	    filtered: []
	};
    // NOTE: _reset() called at the end of the constructor.

    ///
    /// Internal methods.
    ///

    function _reset(){
	card_index['todo'] = [];
	card_index['redo'] = [];
	card_index['done'] = [];
	card_index['filtered'] = [];
	for( var ci = 0; ci < cards.length; ci++ ){
	    card_index['todo'].push(ci);
	}
	// And reset filters.
	positive_filter_list = []; 
    }

    function _deck_exists(str){
	var retval = false;
	if( str == 'todo' || str == 'redo' ||
	    str == 'done' || str == 'filtered' ){
	    retval = true;
	}
	return retval;	
    }

    function _deck_has_card(str){
	var retval = false;
	if( _deck_exists(str) && card_index[str].length > 0 ){
	    retval = true;
	}
	return retval;	
    }

    // In place destructive Fisher–Yates shuffle?
    function _shuffle(list){
	var retval = null;
	
	if( bbop.core.what_is(list) == 'array' ){
	    
	    var l = list.length;
	    if( l == 0 ) return retval;
	    //for( var i = l; i >= 0; i-- ){
	    for( var i = l -1; i >= 0; i-- ){
		var j = Math.floor( Math.random() * l );
		//print(i + ' <=> ' + j);
		var temp = list[i];
		list[i] = list[j];
		list[j] = temp;
	    }		
	    retval = list;
	}
	
	return retval;
    }

    ///
    /// External methods.
    ///

    this.id = function(){ return id; };
    this.name = function(){ return name; };
    this.faces = function(){ return faces_list; };
    this.columns = function(){ return columns_list; };
    
    this.number_of_cards = function(str){
	if( ! str ){ str = 'todo'; }
	var retval = null;
	if( _deck_exists(str) ){
	    retval = card_index[str].length;
	}
	return retval;
    };

    this.is_empty = function(str){
	if( ! str ){ str = 'todo'; }
	return ! _deck_has_card(str);
    };

    // Returns the last card if no offset is given.
    // Returns a cloned object, so have fun.
    this.card_at_offset = function(dname, index){
	var retcard = null;

	// Defaults to dname = 'todo'.
	if( ! dname ){ dname = 'todo'; }
	if( _deck_has_card(dname) ){

	    // Defaults to index = <last_card>.
	    if( ! bbop.core.is_defined(index) ){
		index = card_index[dname].length -1;
	    }
	    
	    // Bounds check.
	    if( index < card_index[dname].length && index >= 0 ){
		
		// Access the real cards!
		var card_ref = card_index[dname][index];
		retcard = bbop.core.clone(cards[card_ref]);
	    }
	}

	return retcard;
    };

    // If no ordered face set is given, no faces are returned.
    this.look_at_card = function(order){
	var retlist = null;

	var cdeck = 'todo';
	if( _deck_has_card(cdeck) ){
	    retlist = [];
	    // Access the real cards!
	    var c = this.card_at_offset(cdeck);
	    each(order,
		 function(oitem){
		     retlist.push(c[oitem]);    
		 });
	}

	return retlist;
    };

    // Return a tag list for the top card. If no column designation is
    // given, all tags are returned. Very similar to look_at_card().
    this.look_at_tags = function(column){
	var retlist = null;

	var cdeck = 'todo';
	if( _deck_has_card(cdeck) ){
	    retlist = [];
	    // Access the real cards!
	    var c = this.card_at_offset(cdeck);
	    var ctags = c['_tags'];
	    each(ctags,
		 function(tname, tval){
		     if( ! bbop.core.is_defined(column) ){
			 retlist.push(tname);
		     }else if( tval == column ){
			 retlist.push(tname);
		     }
		 });
	}

	return retlist;
    };

    // Return the currently held positive filters.
    this.positive_filter_list = function(){
	return positive_filter_list;
    };
	
    // If a card does not have a tag in the list of tags then it is moved
    // to the filtered list. If the list is empty, nothing is
    // filtered.
    // Slow?
    this.apply_positive_filters = function(list_of_tags){
	
	var each = bbop.core.each;
	var is_defined = bbop.core.is_defined;
	var is_empty = bbop.core.is_empty;
	var anchor = this;

	// Add the new filters to the global tracking list.
	each(list_of_tags,
	     function(tag_to_add){
		 positive_filter_list.push(tag_to_add);
	     });

	if( is_defined(list_of_tags) && ! is_empty(list_of_tags) ){

	    // Iterate over all the the decks, except
	    // 'filtered', looking for cards to move.
	    each(['todo', 'redo', 'done'],
		 function(dname){
		     
		     if( _deck_has_card(dname) ){
			 
			 // Cycle down through the cards, splicing
			 // out the ones that we want to get rid
			 // of as we go.
			 //print('dname: ' + dname);
			 //print('cards[dname]: ' + card_index[dname]);
			 //print('cards[dname].length: ' + card_index[dname].length);
			 for(var di = card_index[dname].length -1; di>=0; di--){
			     //print('di: ' + di);
			     var c = anchor.card_at_offset(dname, di);
			     var ctags = c['_tags'];

			     // For each of the tags in our incoming list...
			     var matching_tags = 0;
			     each(list_of_tags,
				  function(tg){
				      if(bbop.core.is_defined(ctags[tg])){
					  matching_tags++;
				      }
				  });

			     if( matching_tags > 0 ){
				 // Something's there, so leave it alone.
				 //print('di: skip');
			     }else{
				 // Nothings there, so splice it out
				 // to the 'filtered' deck.
				 var removed_slot =
				     card_index[dname].splice(di,1);
				 var filtered_item = removed_slot[0];
				 //print('di: remove: ' + filtered_item);
				 card_index['filtered'].push(filtered_item);
			     }
			 }
		     }
		});
	}
    };

    // Fan the cards on the table: list of lists (for debug).
    this.fan = function(dname){
	if( ! dname ){ dname = 'todo'; }

	// Cycle through all the cards.
	var retlist = [];
	each(card_index[dname],
	     function(index){
		 //print('index:' + index);
		 var c = cards[index];

		 // Cycle through all the faces.
		 var minbuf = [];
		 each(columns,
		      function(col){
			  if( col['type'] == 'face' ){
			      //print('col:' + col['name']);
			      minbuf.push(c[col['name']]);
			  }
		      });
		 
		 retlist.push(minbuf); 
	     });
	
	return retlist;
    };

    // Show the reference indices of the deck in question (for debug).
    this.indices = function(dname){
	if( ! dname ){ dname = 'todo'; }
	    
	// Cycle through all the card references.
	var retlist = [];
	each(card_index[dname],
	     function(cref){
		 retlist.push(cref);
	     });
	
	return retlist;
    };

    // Return a list of tag fields available for the deck.
    this.tag_field_list = function(){
	    
	// Cycle through all the card references.
	var retlist = [];
	each(tag_fields,
	     function(tag_field){
		 retlist.push(tag_field);
	     });
	
	return retlist;
    };

    // Return a list of tags available for the deck.
    this.tag_list = function(){
	    
	// Cycle through all the card references.
	var retlist = [];
	each(tags,
	     function(tag){
		 retlist.push(tag);
	     });
	
	return retlist;
    };

    // Move the "top" card from one deck to another deck. False if
    // there is nothing to move.
    this.move_to = function(from_deck, to_deck){
	var retval = false;

	if( _deck_has_card(from_deck) ){
	    var tmp = card_index[from_deck].pop();
	    card_index[to_deck].push(tmp);
	    retval = true;
	}

	return retval;
    };

    // 
    this.redo = function(){
	return this.move_to('todo', 'redo');
    };

    // 
    this.done = function(){
	return this.move_to('todo', 'done');
    };

    //
    this.merge_into = function(deck_source, deck_target){
	var retval = false;

	if( _deck_exists(deck_source) && _deck_exists(deck_target) ){
	    // Bring the source in.
	    card_index[deck_target] = 
		card_index[deck_target].concat(card_index[deck_source]);
	    // Reset the source.
	    card_index[deck_source] = [];
	}

	return retval;
    };

    // Shuffle one of the decks.
    this.shuffle = function(deck){
	if( ! deck ){ deck = 'todo'; }

	var retval = false;

	if( _deck_has_card(deck) ){
	    //print('l1: ' + card_index[deck]);
	    var list = card_index[deck];
	    card_index[deck] = _shuffle(list);
	    //print('l2: ' + card_index[deck]);
	    retval = true;
	}

	return retval;
    };

    // Back to an original "unshuffled" state.
    this.reset = function(){
	_reset();
	//_shuffle();
    };

    ///
    /// Initialization (post def).
    ///

    _reset();
    //_shuffle(); // nevermind
}
