////
//// quiz.js
////
//// Assume the existance of the globals:
////    wrenshow. ...
////    .current_deck
////    .deck_list
////    etc...
////
//// To make this work, there were some coding problems.
//// Apparently, JS does not really do UTF-8, making some things
//// a hideous pain. See:
////  http://mathiasbynens.be/notes/javascript-encoding
//// To get around this, once the data has been been pulled in from the utf-8
//// sources (editted in emacs), the output is in utf-16, which
//// apparently is more digestable (at least my characters are coming
//// out as not garbage).
//// See ../compile-directory.pl for more details.
//// 

// For debugging.
var _logger = new bbop.logger('Q: ');
//_logger.DEBUG = true;
_logger.DEBUG = false;
function _ll(str){
    _logger.kvetch(str);
}
    
///
/// Helper data functions.
///

// TODO: Make less dumb.
// Create a list from the wrenshoe.data array that we should
// have in our environment.
function _get_deck_list(deck_list){
    var found_decks = [];
    bbop.core.each(deck_list,
		   function(datum, index){
		       var d = {
			   id: datum['id'],
			   name: datum['name'],
			   index: index 
		       };
		       found_decks.push(d);
		   });
    return found_decks;
}

function _get_deck(deck_list, deck_id){
    
    var retdeck = null;

    bbop.core.each(deck_list,
		   function(deck_to_check){
		       if( deck_id == deck_to_check['id'] ){
			   retdeck = deck_to_check;
		       }
		   });
    
    // If we found it, convert it to our Deck.
    if( retdeck ){
	retdeck = new Deck(retdeck);
    }

    return retdeck;
}

function _get_first_deck(deck_list){

    var retdeck = null;

    // Something there?
    if( deck_list && deck_list.length && deck_list[0] ){
	retdeck = deck_list[0];
    }
    
    // If we found it, convert it to our Deck.
    if( retdeck ){
	retdeck = new Deck(retdeck);
    }

    return retdeck;
}

function _get_checked_values(checkbox_name){
    var checked_vals = [];
    jQuery("[name='" + checkbox_name + "']:checked").each(
	function(){
	    checked_vals.push(jQuery(this).val());
	});
    return checked_vals;
}

function _add_button(id, bid, blabel, callback){
    jQuery(id).append('<button id="' + id + '" data-role="button" ' +
		      'data-inline="true">' + blabel + '</button>');
    jQuery(bid).click(callback);
}

function _draw_deck_faces_at(deck, faces, target_id){
    //_ll('ddfa');

    //var tid = '#' + target_id;
    var tid = target_id;

    // Clear quiz areas.
    jQuery(tid).empty();

    //
    var str = '';

    // // Start of new bits to try and get things a little more
    // // centered--so distracting on a computer.
    // str += '<div style="text-align:center;">';
    // str += '<div style="display:inline-block; text-align:left;">';

    // First, determine if and what tags to show.
    // This shows all tags in natural order.
    if( wrenshoe.current_meta_show_p ){

	// Determine tag information, if any.
	var tag_line = '';
	var ctags = deck.look_at_tags();
	if( ctags && ctags.length > 0 ){
	    tag_line = '<br />tags: ' + ctags.join(', ');
	}

	// Give a status update.
	str += '<p><small>' +
	    'todo: ' + deck.number_of_cards('todo') +
	    ', redo: ' + deck.number_of_cards('redo') +
	    ', done: ' + deck.number_of_cards('done') +
	    ', filtered: ' + deck.number_of_cards('filtered') +
	    tag_line +
	    '</small></p>';
    }

    // Display whatever we have in the content section.
    var view = deck.look_at_card(faces);
    bbop.core.each(view,
		   function(vitem){
		       str += '<p><big><b>' + vitem + '</b></big></p>';
		   });

    // str += '</div>';
    // str += '</div>';

    jQuery(tid).append(str);
}

///
/// Main page control fuunctions.
///

function QuizInit(){
    _ll('QuizInit');

    ///
    /// Things that we do no matter what.
    ///

    // Hide hideables.
    jQuery(".hideable").hide();

    // Make the select all and unselect all buttons active on the
    // faces page.
    jQuery("#select-all-tags").click(
	function(){
            jQuery('input[name=tag-positive-input]').attr('checked', 'checked');
            jQuery('input[name=tag-positive-input]').checkboxradio('refresh');
	});
    jQuery("#unselect-all-tags").click(
	function(){
            jQuery('input[name=tag-positive-input]').removeAttr('checked');
            jQuery('input[name=tag-positive-input]').checkboxradio('refresh');
	});

    // The redirects always go to the welcome page.
    jQuery(".redirect").click(function(){
				  anywhere2welcome();
			      });
    
    // The home buttons also always go to the welcome page.
    jQuery(".go-home").click(function(){
				  anywhere2welcome();
			      });
    
    // // ... 
    // if( ! wrenshoe.current_deck ){
    // 	_ll('Deck is not defined...');
    // 	// TODO/BUG: Does not work...badly.
    // 	//anywhere2welcome();
    // }

    // Okay, special handling depending on whether or not there are
    // multiple decks or just a single one.
    if( ! wrenshoe.deck_list || ! wrenshoe.deck_list.length ){
	_ll('Global deck list is not well defined...very bad!');
    }else{
	var deck_count = wrenshoe.deck_list.length;
	_ll('There are ' + deck_count + ' decks defined.');

	if( deck_count == 1 ){
	    _ll('Using single mode.');
	    
	    // We actually short-circut go2select behavior.
	    jQuery(".select2review").click(
		function(){
		    
		    // Essentially mimic and
		    // bypass the actions
		    // anywhere2select.
		    jQuery(".redirect").hide();
		    jQuery(".hideable").show();
		    wrenshoe.current_deck = _get_first_deck(wrenshoe.deck_list);
		    anywhere2review();
		});
	    // We actually short-circut go2select behavior.
	    jQuery(".go2select").click(
		function(){
		    
		    // Essentially mimic and
		    // bypass the actions
		    // anywhere2select.
		    jQuery(".redirect").hide();
		    jQuery(".hideable").show();
		    wrenshoe.current_deck = _get_first_deck(wrenshoe.deck_list);
		    anywhere2faces();
				       });
	}else{
	    _ll('Using multi mode.');

	    // Standard behavior.
	    jQuery(".go2select").click(
		function(){
		    anywhere2select();
		});
	}
    }
}

function anywhere2welcome(){ 
    _ll('anywhere2welcome');
    if( jQuery &&
	jQuery.mobile && jQuery.mobile.changePage &&
	jQuery('#welcome') ){
	jQuery.mobile.changePage("#welcome", "slide");	  
    }
}

function anywhere2select(){ 
    _ll('anywhere2select');
   
    var each = bbop.core.each;

    // If we got here legitimately, we can hide all of the redirects
    // and show the normal content.
    jQuery(".redirect").hide();
    jQuery(".hideable").show();

    // First, create a list from the wrenshoe.data array that we should
    // have in our environment.
    var dlist = _get_deck_list(wrenshoe.deck_list);

    // If we found any data, make a list, otherwise write a sad
    // message.
    if( dlist.length == 0 ){
	_ll('data list empty');
	//jQuery('#select-list').append('<b>Hrm. Sorry, no data was found!</b>');
    }else{

	// We have data, so let's add it to our world.
	//jQuery('#select-list').append('<b>Yay--data was found!</b>');
	jQuery('#select-list').empty();
	each(dlist,
	     function(datum, index){
		 // var lstrs = '';
		 // lstrs += '<li class="select2faces" quiz_id="' +
		 //     datum['id'] + '">';
		 // lstrs += datum['name'];
		 // lstrs += '</li>';
		 // jQuery('#sl').append(lstrs);//.listview();

		 var fstr = '';
		 fstr += '<label><input type="radio" name="select-input" value="' + datum['id'] + '" ';
		 if( index == 0 ){ fstr += 'checked="checked" '; }
		 fstr += '/>';
		 fstr += datum['name'];
		 fstr += '</label>';
		 jQuery('#select-list').append(fstr);	 
	     });

	// I don't understand why some of these work and some don't...
	//jQuery('#select-list').listview('refresh');
	//jQuery('#select-list').trigger('refresh');
	jQuery('#select-list').trigger('create');

	jQuery(".select2review").click(
	    function(e){
		var vals = _get_checked_values('select-input');
		if( ! vals || ! vals[0] ){
		    alert('Clicked, but no value found to review!');
		}else{
		    var quiz_id = vals[0];
		    wrenshoe.current_deck =
			_get_deck(wrenshoe.deck_list, quiz_id);
		    // Go...
		    anywhere2review();
		}
	    });
	jQuery(".select2faces").click(
	    function(e){
		var vals = _get_checked_values('select-input');
		if( ! vals || ! vals[0] ){
		    alert('Clicked, but no value found to review!');
		}else{
		    var quiz_id = vals[0];
		    wrenshoe.current_deck =
			_get_deck(wrenshoe.deck_list, quiz_id);
		    // Go...
		    anywhere2faces();
		}
	    });
    }

    // We changes the area, so update the widget.
    //jQuery('#select-list').trigger('updatelayout');
    //jQuery('#select').trigger('updatelayout');
    //jQuery('#foo').trigger('create');
    //jQuery('#foo').listview('refresh');

    // Finally, go to the next page.
    jQuery.mobile.changePage("#select", "slide");
}

function anywhere2review(){
    _ll('anywhere2review');
    //_ensure_deck();

    var each = bbop.core.each;
    var deck = wrenshoe.current_deck;

    if( ! deck ){
	_ll('error get_deck');
    }else{
	
	// Clear the current contents of the table.
	jQuery('#review-table').empty();

	// Group all tags under one heading...for now.
	var cols = bbop.core.clone(deck.faces());
	cols.push('Tags');
	var lcache = [];
	while( ! deck.is_empty() ){
	    
	    var writes = bbop.core.clone(deck.look_at_card(deck.faces()));
	    writes.push(deck.look_at_tags().join(', '));
	    
	    lcache.push(writes);

	    // Push stop condition.
	    deck.done();
	}

	lcache.reverse();
	var tbl = new bbop.html.table(cols, lcache);//, deck.fan());

	jQuery('#review-table').append(tbl.to_string());

	// Transition to new page.
	jQuery.mobile.changePage("#review", "slide");
    }
}

function anywhere2faces(){
    _ll('anywhere2faces');
    //_ensure_deck();

    var each = bbop.core.each;
    var deck = wrenshoe.current_deck;

    if( ! deck ){
	_ll('error get_deck');
    }else{
	
	// Clear the current contents of faces.
	jQuery('#front-faces').empty();
	jQuery('#back-faces').empty();

	// Build up the front and back.
	var faces = deck.faces();

	// Front faces.
	each(faces,
	     function(face, index){
		 var fstr =
		     '<label><input type="checkbox" name="front-face-input" value="' + face + '" class="custom" ';
		 if( index == 0 ){ fstr += 'checked="checked" '; }
		 fstr += '/>';
		 fstr += face;
		 fstr += '</label>';
		 jQuery('#front-faces').append(fstr);
	     });
	jQuery('#front-faces').trigger('create');

	// Back faces.
	each(faces,
	     function(face, index){
		 var fstr =
		     '<label><input type="checkbox" name="back-face-input" value="' + face + '" class="custom" ';
		 //if( index == 0 ){ fstr += 'checked="checked" '; }
		 fstr += 'checked="checked" ';
		 fstr += '/>';
		 fstr += face;
		 fstr += '</label>';
		 jQuery('#back-faces').append(fstr);
	     });
	jQuery('#back-faces').trigger('create');

	// Now make the tag are clear and build it up.
	jQuery('#tags-faces').empty();
	var tags = deck.tag_list();
	if( ! tags || ! tags.length || tags.length == 0 ){
	    // If no tags are defined, take out the whole section.
	    jQuery('.no-tag-destroy').hide();
	}else{
	    // Otherwise, we're going to build up the inputs.
	    jQuery('.no-tag-destroy').show();
	    tags.sort(); // just this sort for now
	    each(tags,
		 function(tag){
		     var fstr =
			 '<label><input type="checkbox" name="tag-positive-input" value="' + tag + '" class="custom" ';
		     fstr += '/>';
		     fstr += tag;
		     fstr += '</label>';
		     jQuery('#tags-faces').append(fstr);
		 });
	    jQuery('#tags-faces').trigger('create');
	}

	// Make the "Next" button live.
	jQuery('.faces2loop').click(
	    function(e){
		
		// Get the faces that we'll use.
		wrenshoe.current_front_faces =
		    _get_checked_values('front-face-input');
		wrenshoe.current_back_faces =
		    _get_checked_values('back-face-input');
		wrenshoe.current_filters =
		    _get_checked_values('tag-positive-input');

		// Also get the "optional" show tags flag.
		var what_is = bbop.core.what_is;
		var is_defined = bbop.core.is_defined;
		var tsi_list = _get_checked_values('meta-show-input');	
		if( is_defined(tsi_list) && what_is(tsi_list) == 'array' &&
		    tsi_list.length == 1 ){
			var tsi = tsi_list[0];
			if( what_is(tsi) == 'string' && tsi == 'show' ){
			    wrenshoe.current_meta_show_p = true;
			}
		}
		
		// Shuffle once before we go in.
		wrenshoe.current_deck.reset();
		wrenshoe.current_deck.shuffle();

		// Apply any filters we might have just picked up.
		var pf = wrenshoe.current_filters;
		wrenshoe.current_deck.apply_positive_filters(pf);

		//_ll(bbop.core.dump(selected_front_faces));
		anywhere2front();
	    });

	// Transition to new page.
	jQuery.mobile.changePage("#faces", "slide");
    }
}

function anywhere2front(){
    _ll('anywhere2front');
    //_ensure_deck();

    var cid = "#current-card-content-front";
    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_front_faces;

    // Deck emptiness should be a front-only problem. Bump to results.
    // If there are no cards.
    if( deck.is_empty() ){
	_ll('anywhere2front: deck is empty');
	jQuery(cid).append('EMPTY');
	anywhere2results();	
    }else{

	_draw_deck_faces_at(deck, faces, cid);

	// Clear listeners and reactivate buttons.
	jQuery('#front-okay').unbind('click');
	jQuery('#front-okay').click(
	    function(){
		anywhere2back();
	    }
	);

	jQuery.mobile.changePage("#loop-front", "slide");
    }
}


// Main looping function on the front.
function anywhere2back(){
    _ll('anywhere2back');
    //_ensure_deck();

    var cid = "#current-card-content-back";
    var deck = wrenshoe.current_deck;
    var faces = wrenshoe.current_back_faces;

    _draw_deck_faces_at(deck, faces, cid);

    // Clear listeners and reactivate buttons.
    jQuery('#back-yes').unbind('click');
    jQuery('#back-yes').click(
	function(){
	    deck.done();
	    anywhere2front();
	}
    );
    jQuery('#back-no').unbind('click');
    jQuery('#back-no').click(
	function(){
	    deck.redo();
	    anywhere2front();
	}
    );

    jQuery.mobile.changePage("#loop-back", "slide");
    // // Refresh the view.
    // jQuery('#loop').trigger('refresh');
}


// Main looping function on the front.
function anywhere2results(){
    _ll('anywhere2results');
    //_ensure_deck();

    var deck = wrenshoe.current_deck;

    if( ! deck ){
	alert('ERROR: You got to results here without a deck!');
	throw new Error('ERROR: You got to results here without a deck!');
    }

    // ...
    var rid = '#results-content';
    jQuery(rid).empty();
    jQuery(rid).append('<p>todo: ' + deck.number_of_cards('todo') + '</p>' +
		       '<p>redo: ' + deck.number_of_cards('redo') + '</p>' +
		       '<p>done: ' + deck.number_of_cards('done') + '</p>' +
		       '<p>filtered: '+deck.number_of_cards('filtered')+'</p>');
	
    // Clear out old event listeners.
    jQuery('#results-reselect').unbind('click');
    jQuery('#results-reselect').click(
	function(){	    
	    anywhere2select();
	}
    );
    jQuery('#results-again').unbind('click');
    jQuery('#results-again').click(
	function(){	    
	    var pa = deck.positive_filter_list();
	    deck.reset();
	    deck.shuffle();
	    deck.apply_positive_filters(pa);
	    anywhere2front();
	}
    );
    jQuery('#results-redo').unbind('click');
    jQuery('#results-redo').click(
	function(){	    
	    deck.merge_into('redo', 'todo');
	    deck.shuffle();
	    anywhere2front();
	}
    );

    // ...and go.
    jQuery.mobile.changePage("#results", "slide");
}

// Go!
jQuery(document).on("mobileinit", function(){
			// 'fade' was too slow for a fast quiz.
			// And even slower on mobile browsers.
			jQuery.mobile['defaultPageTransition'] = 'none';
});
// Let's just wait until everything is settled. Just once.
jQuery(document).ready(function(){ QuizInit(); });
// // Ah! This one was for every page.
//jQuery(document).live('pageinit', function(){ QuizInit(); });
