####
#### Super set and emitter for decks.
####

package Wrenshoe::Page;

use strict;
use utf8;
use Data::Dumper;
use Template;

use Wrenshoe::Utilities qw(:all);
use Wrenshoe::Error;
use Wrenshoe::Deck;
use Wrenshoe::Compile;

## Given a loaded DeckSet, produce a page given an environment.
sub new {

  ##
  my $class = shift;
  #my $ds = shift || die "need a DeckSet arg"
  my $ds = shift || undef; # could be static
  my $count = shift || undef; # only used if $ds is not defined

  my $self = {};

  ## Use either the DeckSet or the given argument to figure out
  ## whether we are in a single or multi environment.
  $self->{deckset} = $ds;
  $self->{count} = $count;
  if( ! $self->{deckset} ){
    if( ! $self->{count} ){
      WrenshoeError->throw(message=>
			   "count must be defined if DeckSet not present");
      # }else{
      #   $self->{count} = $count;
    }
  }else{
    if( ! $self->{deckset}->count() ){
      WrenshoeError->throw(message=>"DeckSet has no decks defined");
    }else{
      $self->{count} = $self->{deckset}->count();
    }
  }

  $self->{template_paths} = ['template'];

  $self->{static_source_path} = '';
  $self->{google_analytics_id} = '';
  $self->{page_title} = 'Wrenshoe';
  $self->{page_description} = '';

  $self->{inline_data_p} = 0;

  bless $self, $class;
  return $self;
}

sub static_source_path {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'static_source_path', $arg);
}

sub google_analytics_id {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'google_analytics_id', $arg);
}

sub title {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'page_title', $arg);
}

sub description {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'page_description', $arg);
}

sub inline_data_p {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'inline_data_p', $arg);
}

sub template_paths {
  my $self = shift;
  my $arg = shift || undef;
  return getter_setter($self, 'template_paths', $arg);
}

sub page {
  my $self = shift;
  my $pretty_p = shift || 0;

  ###
  ### Assemble the parameters that we'll need.
  ###

  my $params = {};

  ## Decide if it will be a single or multipage setup.
  if( ! $self->{count} ){ # should have been caught already
      WrenshoeError->throw(message=>"page production error");
  }elsif( $self->{count} == 1 ){
    $params->{MULTI_P} = 0;
    $params->{SINGLE_P} = 1;
  }else{
    $params->{MULTI_P} = 1;
    $params->{SINGLE_P} = 0;
  }

  ## GA.
  $params->{GOOGLE_ANALYTICS_ID} = $self->google_analytics_id();

  ## Set parameters that we're not going to use.
  $params->{STATIC_PATH} = $self->static_source_path();
  $params->{QUIZ_TITLE} = $self->title();
  $params->{QUIZ_DESCRIPTION} = $self->description();

  ## Switch between using external and set static JS or generating our
  ## own right here with the page write-out.
  $params->{DATA_STRING} = '';
  if( $self->inline_data_p() ){
    my $data = $self->{deckset}->json($pretty_p);
    my @out = ();
    push @out, 'var wrenshoe = {};';
    push @out, 'wrenshoe.current_deck = null;';
    push @out, 'wrenshoe.current_front_faces = [];';
    push @out, 'wrenshoe.current_back_faces = [];';
    push @out, 'wrenshoe.current_filters = [];';
    push @out, 'wrenshoe.current_meta_show_p = false;';
    push @out, 'wrenshoe.deck_list = ' . $data . ';';
    $params->{DATA_STRING} = join("\n", @out);
  }

  ###
  ### Page output.
  ###

  my $tt_paths = $self->template_paths();
  my $tt_params =
    {
     INCLUDE_PATH => join(':', @$tt_paths),
     #INTERPOLATE  => 1,
    };
  my $tt = Template->new($tt_params)
    || die "$Template::ERROR\n";
  my $ret = '';
  $tt->process('wrenshoe.tt', $params, \$ret)
    || die $tt->error(), "\n";

  return $ret;
}



1;
