####
#### A simple set of utilities that we'll use around here.
####
#### A static function collection takes a little more monkeying around
#### to get nicely useable.
####

package Wrenshoe::Utilities;

use strict;
use Exporter qw(import);
use utf8;
use Data::Dumper;
use File::Slurp;

our $VERSION = 1.00;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(ll out err dequote splode strip getter_setter slurp_to_string slurp_to_lines);
our %EXPORT_TAGS =
  (
   all => [qw(ll out err dequote splode strip getter_setter slurp_to_string slurp_to_lines)]
  );

our $DEBUG = 0;

sub ll {
  my $arg = shift || '';
  print STDERR $arg . "\n" if $DEBUG;
}

sub out {
  my $arg = shift || '';
  print STDOUT $arg . "\n";
}

sub err {
  my $arg = shift || '';
  print STDERR $arg . "\n";
  die "$arg";
}

sub dequote {
  my $arg = shift || '';
  my $ret = $arg;
  if( $ret =~ /^\"(.*)\"$/ ){
    $ret = $1;
  }
  return $ret;
}

## Okay, splode must be a little subtle here, as perl is odd in this
## area: http://www.perlmonks.org/?node_id=72247
## The naive code:
##
## : my $arg = shift || '';
## : my $cut_on = shift || "\t";
## : my @ret = split($cut_on, $arg);
## : return @ret;
##
## Will actually give the wrong value in some cases, since, I believe
## the final undef is getting cut. Also see the "pre-splode" test
## cases in the unit tests.
## Oh--just the -1.
sub splode {
  my $arg = shift || '';
  my $cut_on = shift || "\t";
  #print STDERR "<<<arg: (" . $arg . ")\n";
  #print STDERR "<<<cut_on: (" . $cut_on . ")\n";
  my @ret = split($cut_on, $arg, -1);
  #print STDERR ">>>" . scalar(@ret) . "\n";
  #print STDERR ">>>" . ($#ret + 1) . "\n";
  #print STDERR ">>>(" . join(')(', @ret) . ")\n";
  return @ret;
}

sub strip {
  my $arg = shift || '';
  my $strp = shift || "\\s";

  my $frgxp = '^' . $strp . '+';
  my $lrgxp = '' . $strp . '+$';
  #err(':: ' . $frgxp);
  $arg =~ s/$frgxp//;
  $arg =~ s/$lrgxp//;

  return $arg;
}

sub getter_setter {
  my $object = shift || die "need object arg";
  my $target = shift || die "need target arg";
  my $value = shift || undef;
  if( defined($value) ){
    $object->{$target} = $value;
  }
  return $object->{$target};
}

sub slurp_to_string {
  my $arg = shift || die 'need a filename arg';
  my $txt = read_file($arg);
  return $txt;
}

sub slurp_to_lines {
  my $arg = shift || die 'need a filename arg';
  my $txt_lines = read_file($arg, array_ref => 1, chomp => 1);
  return $txt_lines;
}


1;
