use strict;
use Test::More 'no_plan';
use Data::Dumper;

use Wrenshoe::Utilities qw(:all);
use Wrenshoe::Deck;

## foo
## bar
## face tag
## f1 t1
## a b
## c d, b
my $deck = Wrenshoe::Deck->new();
ok($deck->okay(), 'nothing so fine');

$deck->name('foo');
$deck->description('bar');
ok($deck->name() eq 'foo', 'name');
ok($deck->description() eq 'bar', 'desc');

is($deck->add_face('f1', 'face'), 1, 'one face');
ok($deck->okay(), 'added faces 1');
is($deck->add_face('t1', 'tag'), 2, 'two faces');
ok($deck->okay(), 'added faces 2');

## Add a "bad" card--short two fields--but depend on right-fill
## behavior. Right-fill will not cover for the missing but required
## tag.
is($deck->add_card([]), 0, '"bad" card not added 1');
ok(! $deck->okay(), '"bad" card has no tag');
ok($deck->error_type() eq 'tag', 'yup, bad tag');
#err('msg: ' . $deck->error_message());
$deck->reset_error();

## Check reset.
ok($deck->okay() , 'reset okay');

## Add another bad card--short one field.
is($deck->add_card(['a']), 0, '"bad" card not added 2');
ok(! $deck->okay(), '"bad" card has no tag 2');
ok($deck->error_type() eq 'tag', 'yup, bad tag 2');
$deck->reset_error();

## Add a bad card--one too many (3) fields.
is($deck->add_card(['a', 'b', 'c']), 0, 'bad card nothing added 3');
ok(! $deck->okay() , 'bad card not okay 3');
$deck->reset_error();

## Okay, first good card.
is($deck->add_card(['a', 'b']), 1, 'good card added 1');
ok($deck->okay() , 'good card okay 1');

## Second good card.
is($deck->add_card(['c', 'd, b']), 2, 'good card added 2');
ok($deck->okay() , 'good card okay 2');

## Probe the data structures.
is_deeply($deck->get_columns(),
	  [
	   {
	    'name' => 'f1',
	    'type' => 'face'
	   },
	   {
	    'name' => 't1',
	    'type' => 'tag'
	   }
	  ],
	  'deep faces');
is_deeply($deck->get_tags(),
	  {
	   'b' => 2,
	   'd' => 1,
	  },
	  'deep tags');
is_deeply($deck->get_tag_fields(),
	  {
	   't1' => 3,
	  },
	  'deep tags fields');
is_deeply($deck->get_cards(),
	  [
	   {
	    '_tags' => {
			'b' => 't1'
		       },
	    'f1' => 'a'
	   },
	   {
	    '_tags' => {
			'b' => 't1',
			'd' => 't1'
		       },
	    'f1' => 'c'
	   }
	  ],
	  'deep cards');
