use strict;
use Test::More 'no_plan';
use Data::Dumper;

#use Wrenshoe::Utilities qw(ll out err dequote splode strip getter_setter);
use Wrenshoe::Utilities qw(:all);
#use Wrenshoe::Utilities;

## dequote
ok( dequote('"foo"') eq 'foo', 'dequote 1');
ok( dequote('foo') eq 'foo', 'dequote 2');
ok( dequote('foo"') eq 'foo"', 'dequote 3');
ok( dequote('"foo') eq '"foo', 'dequote 4');

## pre-splode
my $foo = [];
is(scalar(@$foo), 0, 'pre-splode sanity 1');
is(scalar(qw(1)), 1, 'pre-splode sanity 2');
is(scalar(qw(1 2 3)), 3, 'pre-splode sanity 3');
ok(undef == '', 'pre-splode sanity 4'); # some of the splode problem is here
ok(undef eq '', 'pre-splode sanity 5'); # some of the splode problem is here

## splode
my @s1 = splode("a");
ok( $s1[0] eq 'a', 'splode 1');
my @s2 = splode("a\tb");
ok( $s2[0] eq 'a', 'splode 2');
ok( $s2[1] eq 'b', 'splode 3');
my @s3 = splode("a, b", ",");
is(scalar(@s3), 2, "splode 4");
ok( $s3[0] eq 'a', 'splode 5');
ok( $s3[1] eq ' b', 'splode 6');
my @s4 = splode("\ta\t");
is(scalar(@s4), 3, "splode 7");
ok( $s4[0] eq '', 'splode 8');
ok( $s4[1] eq 'a', 'splode 9');
ok( $s4[2] eq '', 'splode 10');
my @s5 = splode("1	2	3	");
#my @s5 = splode("1\t2\t3\t");
is(scalar(@s5), 4, "splode 11");
ok( $s5[0] eq '1', 'splode 12');
ok( $s5[1] eq '2', 'splode 13');
ok( $s5[2] eq '3', 'splode 14');
ok( $s5[3] eq '', 'splode 15');

## strip
ok( strip('foo') eq 'foo', 'strip 1');
ok( strip('foo  ') eq 'foo', 'strip 2');
ok( strip(' foo  ') eq 'foo', 'strip 3');
ok( strip('  foo') eq 'foo', 'strip 4');
ok( strip("  foo \t") eq 'foo', 'strip 5');
ok( strip("  foo \t", ' ') eq "foo \t", 'strip 6');

## getter_setter
my $o1 = {};
ok( getter_setter($o1, 'foo') == undef, 'getter_setter 1');
ok( getter_setter($o1, 'foo', 'bar') eq 'bar', 'getter_setter 2');
ok( $o1->{foo} eq 'bar', 'getter_setter 3');
ok( getter_setter($o1, 'foo') eq 'bar', 'getter_setter 4');
