#!/usr/bin/perl -w
####
#### Build the static site in staging.
####
#### Test run:
####  perl -I perl/lib ./scripts/build-static-jqui-page.pl --verbose --google-analytics='' --data=example-multi
####
####

use strict;
use utf8;
use Data::Dumper;
use Getopt::Long;
use Template;
use Wrenshoe::Utilities qw(:all);
use Wrenshoe::Page;
use Wrenshoe::Bootstrap;
use Wrenshoe::DeckSet;

binmode STDOUT, ":utf8";
binmode STDERR, ":utf8";

###
### Flags.
###

my $verbose = '';
my $help = '';
my $google_analytics = 'null';
my $data = '';
my $url = '';
my $title = '';
my $description = '';
GetOptions ('verbose' => \$verbose,
	    'help' => \$help,
	    'google-analytics=s' => \$google_analytics,
	    'data=s' => \$data,
	    'url=s' => \$url,
	    'title=s' => \$title,
	    'description=s' => \$description);

if( $verbose ){
  $Wrenshoe::Utilities::DEBUG = 1;
}

## Get the handle ready.
my $pgen = undef;

## Assuming we have the data, we just need to get a meta-count and
## intialize $pgen.
if( ! $data && ! $url ){
  err("Need to set --data or --url.");
}elsif( $data ){
  ll("Will look at data in: $data.");

  ## Count the number of meta files to use in the constructor.
  my @meta_files = <$data/*.meta>;
  my $meta_found_count = scalar(@meta_files);
  if( ! $meta_found_count ){ # includes 0
    err("Directory $data seems to have no data files.");
  }elsif( $meta_found_count == 1 ){
    ll("Found $meta_found_count data file: will use SINGLE mode.");
  }else{
    ll("Found $meta_found_count data files: will use MULTI mode.");
  }

  $pgen = Wrenshoe::JQUIPage->new(undef, $meta_found_count);

  ## Get the desc and title info.
  my $title_template_file = $data . '/title.html';
  my $description_template_file = $data . '/description.html';
  ## Make sure it's there.
  if( ! -f $title_template_file ){
    err("Cannot find required $title_template_file.");
  }elsif( ! -f $description_template_file ){
    err("Cannot find required $description_template_file.");
  }else{
    ll("Will use data at: $data.");
    ll("Will use title info at: $title_template_file.");
    ll("Will use description info at: $description_template_file.");
  }
  ## While we're here, our directory title and description.
  $pgen->title(slurp_to_string($title_template_file));
  $pgen->description(slurp_to_string($description_template_file));

}else{
  ll("Will take data from: \"$url\"");
  ll("Will use title: \"$title\"");
  ll("Will use description: \"$description\"");

  my $deckset = Wrenshoe::DeckSet->new();
  $deckset->add_google_spreadsheet($url);
  my $meta_count_found = $deckset->count();

  $pgen = Wrenshoe::JQUIPage->new(undef, $meta_count_found);
  $pgen->title($title) if $title;
  $pgen->description($description) if $description;
}

## Google Analytics--same either way.
if( $google_analytics && $google_analytics ne 'null' ){
  $pgen->google_analytics_id($google_analytics);
  ll("Will use Google Analytics with ID: $google_analytics.");
}else{
  ll("Will not use Google Analytics analytics.");
}

## Output setup.
#$pgen->template_paths(['template', $data]);
$pgen->template_paths(['template/js-jqui', 'template/common']);
my $pout = $pgen->page();
out($pout);
