#!/usr/bin/perl -w
####
#### Produce the raw data*.js file--the main dynamic content for the
#### pages.
####
#### Find all the *.wmeta/wdata pairs, or *.wrenshoe files, in a
#### directory; compile them into JSON data structures; output to
#### STDOUT a single JS array with all of them.
####
#### Used as:
####  ./compile-directory.pl -v -d ../data > ../js/data.js
#### Then, don't forget:
####  iconv -f utf-8 -t utf-16 js/data.js > js/data.utf-16.js
####

use strict;
use utf8;
use Data::Dumper;
use Getopt::Std;
use Try::Tiny;

#use lib '../perl/lib';
use Wrenshoe::Utilities qw(:all);
use Wrenshoe::Compile;
use Wrenshoe::DeckSet;

## Debugging?
$Wrenshoe::Utilities::DEBUG = 1;

use vars qw(
             $opt_v
             $opt_d
	  );
getopts('vd:');

#binmode STDOUT, ":utf8";
#binmode STDERR, ":utf8";

my $deckset = Wrenshoe::DeckSet->new();

die "You must supply a data directory with \"-d\"!" if ! $opt_d;
die "Could not find directory: $opt_d" if ! -d $opt_d;

## Glob and load all meta/data file pairs.
my @meta_files = <$opt_d/*.wmeta>;
foreach my $mname (@meta_files){
  my $base = substr $mname, 0, -6;
  #ll($base);

  ## If there is a matching data file, add it to the list.
  my $dname = $base . '.wdata';
  if( -f $dname && -r $dname ){

    try{

      my $comp = Wrenshoe::Compile->new();
      $comp->parse_json_fileset($mname, $dname);
      $deckset->add_deck($comp->deck());

    }catch{
      ## TODO: Use nice templates for this type of more complcated
      ## error.
      if( $_->isa('WrenshoeDataError') ){
	die "There was a " . $_->type . " error in your input.";
      }elsif( $_->isa('WrenshoeCompileError') ){
	err("There was a " . $_->type .
	    " error in your input ($dname or $mname); look at: " . $_->line .
	    " for " . $_->message);
      }elsif( $_->isa('WrenshoeError') ){
	die "There was a generic error in your input.";
      }else{
	die "Unknown error: $_";
      }
    };
  }
}

## Glob and load all wrenshoe files.
my @wren_files = <$opt_d/*.wrenshoe>;
foreach my $wrenname (@wren_files){
  my $comp = Wrenshoe::Compile->new();
  $comp->parse_filename($wrenname);
  $deckset->add_deck($comp->deck());
}

#ll('Set: ' . Dumper($deckset));

###
### Dump out to JSON.
###

# out("var wrenshoe = {};");
# out("wrenshoe.current_deck = null;");
# out("wrenshoe.current_front_faces = [];");
# out("wrenshoe.current_back_faces = [];");
# out("wrenshoe.current_filters = [];");
# out("wrenshoe.current_meta_show_p = false;");
# out("");
#my $outstr = $deckset->json(1);
#$outstr = 'wrenshoe.deck_list = ' . $outstr . ";\n";
my $outstr = $deckset->data_js(1);
out($outstr);
