#!/usr/bin/perl -w
####
#### Produce the raw data*.js file--the main dynamic content for the
#### pages.
####
#### Find the URL and load it.
####
#### Used as:
####  ./compile-data-url-to-js.pl -v -u "http://foo" > ../js/data.js
#### Then, don't forget:
####  iconv -f utf-8 -t utf-16 js/data.js > js/data.utf-16.js
####

use strict;
use utf8;
use Data::Dumper;
use Getopt::Std;
use Try::Tiny;

#use lib '../perl/lib';
use Wrenshoe::Utilities qw(:all);
use Wrenshoe::Compile;
use Wrenshoe::DeckSet;

use vars qw(
             $opt_v
             $opt_u
	  );
getopts('vu:');

binmode STDOUT, ":utf8";
binmode STDERR, ":utf8";

die "You must supply a data URL with \"-u\"!" if ! $opt_u;
my $url = $opt_u;

if( $opt_v ){
  ## Debug.
  $Wrenshoe::Utilities::DEBUG = 1;
}

ll('URL: ' . $url);

my $deckset = Wrenshoe::DeckSet->new();

try {
  $deckset->add_google_spreadsheet($url);
} catch {
  if( $_->isa('WrenshoeCompileError') ){
    die $_->type . " error on line " . $_->line . ": " . $_->message . "\n";
  }elsif( $_->isa('WrenshoeDataError') ){
    die $_->type . " error for " . $_->resource . ": " . $_->message . "\n";
  }elsif( $_->isa('WrenshoeError') ){
    die $_->message . "\n";
  }else{
    #die "error while working: $!";
  }
};
#ll('Set: ' . Dumper($deckset));

my $outstr = $deckset->data_js(1);
out($outstr);
