#!/usr/bin/python
# -*- coding: utf-8 -*-
####
#### Dump the main points of Kansaibenkyou's word database and convert
#### it into json.
####
#### Usage:
####   sjcarbon@moiraine:~/local/src/bazaar/wrenshoe/scripts$:) ./serialize-kb.py > ../example-single/kansaiben.data
####

from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import *
import os
import json
import re
import sys
from jinja2 import Template, Environment, PackageLoader

Base = declarative_base()

class Node(Base):
    __tablename__ = 'node'

    nid = Column(Integer, primary_key=True)
    type = Column(String)
    title = Column(String)
    status = Column(String)

    def __init__(self, nid, type, title, status):
        self.nid = nid
        self.type = type
        self.title = title
        self.status = status

    def __repr__(self):
        return "<Node('%s','%s')>" % (self.nid, self.title)

class WordMeaning(Base):
    __tablename__ = 'field_data_body'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    body_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.body_value = val

    def __repr__(self):
        return "<WordMeaning('%s','%s')>" % (self.entity_id,
                                             self.body_value)

class WordKanji(Base):
    __tablename__ = 'field_data_field_word_kanji'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_kanji_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_kanji_value = val

    def __repr__(self):
        return "<WordKanji('%s','%s')>" % (self.entity_id,
                                           self.field_word_kanji_value)

class WordStandard(Base):
    __tablename__ = 'field_data_field_word_standard'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_standard_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_standard_value = val

    def __repr__(self):
        return "<WordStandard('%s','%s')>" % (self.entity_id,
                                              self.field_word_standard_value)

class WordStandardKanji(Base):
    __tablename__ = 'field_data_field_word_standard_kanji'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_standard_kanji_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_standard_kanji_value = val

    def __repr__(self):
        return "<WordStandardKanji('%s','%s')>" % (self.entity_id,
                                                   self.field_word_standard_kanji_value)

class WordExample(Base):
    __tablename__ = 'field_data_field_word_example'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_example_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_example_value = val

    def __repr__(self):
        return "<WordExample('%s','%s')>" % (self.entity_id,
                                             self.field_word_example_value)

class WordSuitability(Base):
    __tablename__ = 'field_data_field_word_suitability'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_suitability_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_suitability_value = val

    def __repr__(self):
        return "<WordSuitability('%s','%s')>" % (self.entity_id,
                                                 self.field_word_suitability_value)

class WordCommentary(Base):
    __tablename__ = 'field_data_field_word_commentary'

    entity_id = Column(Integer, ForeignKey('node.nid'), primary_key=True)
    field_word_commentary_value = Column(String)

    def __init__(self, id, val):
        self.entity_id = id
        self.field_word_commentary_value = val

    def __repr__(self):
        return "<WordCommentary('%s','%s')>" % (self.entity_id,
                                                self.field_word_commentary_value)

## Some relationships.
Node.word_kanji = relationship(WordKanji, primaryjoin=Node.nid == WordKanji.entity_id)
Node.word_meaning = relationship(WordMeaning, primaryjoin=Node.nid == WordMeaning.entity_id)
Node.word_standard = relationship(WordStandard, primaryjoin=Node.nid == WordStandard.entity_id)
Node.word_standard_kanji = relationship(WordStandardKanji, primaryjoin=Node.nid == WordStandardKanji.entity_id)
Node.word_example = relationship(WordExample, primaryjoin=Node.nid == WordExample.entity_id)
Node.word_suitability = relationship(WordSuitability, primaryjoin=Node.nid == WordSuitability.entity_id)
Node.word_commentary = relationship(WordCommentary, primaryjoin=Node.nid == WordCommentary.entity_id)

##
if __name__ == "__main__":

    ## Database setup.
    #conn = 'mysql+mysqldb://192.168.1.6/kansaibenkyou?charset=utf8'
    conn = 'mysql+mysqldb://localhost/kansaibenkyou?charset=utf8'
    engine = create_engine(conn, pool_recycle=3600)
    Session = sessionmaker()
    Session.configure(bind=engine)
    session = Session()

    ## Iterate through and cache.
    out_data = {}
    for w in session.query(Node).filter_by(type='word', status=1): 
        #print "word", w.nid, w.title
        key = w.title
        out_data[key] = {
            'node_id': w.nid,
            'word': w.title,
            'meaning': None,
            'kanji': None,
            'standard_word': None,
            'standard_kanji': None,
            #'examples': [],
            #'commentary': [],
            'suitability': [],
            }
        
        ## word_meaning: may be one.
        wm_list = w.word_meaning
        if len(wm_list) > 0:
            # for wk in wk_list:
            #     print "\tword_kanji", wk.entity_id, wk.field_word_kanji_value
            out_data[key]['meaning'] = wm_list[0].body_value
            
        ## word_kanji: may be one.
        wk_list = w.word_kanji
        if len(wk_list) > 0:
            # for wk in wk_list:
            #     print "\tword_kanji", wk.entity_id, wk.field_word_kanji_value
            out_data[key]['kanji'] = wk_list[0].field_word_kanji_value
            
        ## word_standard: may be one.
        ws_list = w.word_standard
        if len(ws_list) > 0:
            # for ws in ws_list:
            #     print "\tword_standard", ws.entity_id, ws.field_word_standard_value
            out_data[key]['standard_word'] = ws_list[0].field_word_standard_value

        ## word_standard_kanji: may be one.
        wsk_list = w.word_standard_kanji
        if len(wsk_list) > 0:
            # for wsk in wsk_list:
            #     print "\tword_standard_kanji", wsk.entity_id, wsk.field_word_standard_kanji_value
            out_data[key]['standard_kanji'] = wsk_list[0].field_word_standard_kanji_value

        # ## word_example: may be many.
        # we_list = w.word_example
        # if len(we_list) > 0:
        #     for we in we_list:
        #         #print "\tword_example", we.entity_id, we.field_word_example_value
        #         out_data[key]['examples'].append(we.field_word_example_value)
                
        # ## word_commentary: may be many
        # wc_list = w.word_commentary
        # if len(wc_list) > 0:
        #     for wc in wc_list:
        #         #     print "\tword_commentary", wc.entity_id, wc.field_word_commentary_value
        #         out_data[key]['commentary'].append(wc.field_word_commentary_value)

        ## word_suitability: may be one, but we'll fold it out
        ## We have some field cleaning in there...
        wsuit_list = w.word_suitability
        if len(wsuit_list) > 0:
            #print "\tword_suitability", wsuit.entity_id, wsuit.field_word_suitability_value
            wsuit_str = wsuit_list[0].field_word_suitability_value
            if re.search('older', wsuit_str):
                out_data[key]['suitability'].append('older')
            if re.search('i-female', wsuit_str):
                out_data[key]['suitability'].append('female')
            if re.search('i-male', wsuit_str):
                out_data[key]['suitability'].append('male')
            if re.search('upsetting', wsuit_str):
                out_data[key]['suitability'].append('upsetting')
            if re.search('formal', wsuit_str):
                out_data[key]['suitability'].append('formal')
                
    for k in out_data:
        word = out_data[k]['word'] or 'n/a'
        meaning = out_data[k]['meaning'] or 'n/a'
        kanji = out_data[k]['kanji'] or 'n/a'
        std_w = out_data[k]['standard_word'] or 'n/a'
        std_k = out_data[k]['standard_kanji'] or 'n/a'
        #ex = out_data[k]['examples'] or []
        #cm = out_data[k]['commentary'] or []
        suit = out_data[k]['suitability'] or []
        suit_str = ", ".join(suit) or "general"
        node = '<a href="http://kansaibenkyou.net/node/' + str(out_data[k]['node_id']) + '" title="See ' + word + ' on kansaibenkyou.net">&rarr;</a>'
        #print word + "\t" + meaning + "\t" + kanji + "\t" + std_w + "\t" + std_k + "\t" + suit_str + "\t" + node
        #print word
        #sys.stdout.buffer.write(word)
        #print word.encode('utf8')
        pre_final = word + "\t" + meaning + "\t" + kanji + "\t" + std_w + "\t" + std_k + "\t" + suit_str + "\t" + node

        ## Nuke the ^M newlines into spaces.
        pre_final = pre_final.replace("\r\n", " ")
        #pre_final = pre_final.replace("\r", " ")

        ## Get into decent unicode for output.
        print pre_final.encode('utf8')
        
